function [sys,x0,str,ts]=sfunddeout(t,x,u,flag,service,topic,item,Ts)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [sys,x0,str,ts]=sfunddeout(t,x,u,flag,service,topic,item,Ts)   %
%                                                                         %
% Output of Simulink via DDE communication. Input is a scalar.            %
%                                                                         %
% INPUT:                                                                  %
%       t       Current time                                              %
%       x       DDE comunication chanel handle                            %                              
%       u       Input from simulation                                     %
%       flag    Task to be performed by the S-function                    %
%                                                                         %
% USER INPUT:                                                             %                                                               
%       service String specifying the service for DDE conversation        %          
%       topic   String specifying the topic for  DDE conversation         %
%       item    String specifying the DDE item                            %
%       Ts      Sample time [-1 = Inherited sample time]                  %
%                                                                         %
% OUTPUT:                                                                 %
%       sys     Variable using                                            %
%       x0      DDE comunication chanel handle                            %
%       str     Not use                                                   %                                                 
%       ts      Sample time                                               %                                                 
%                                                                         %
%                                                                         %
% Date of last change: 13.10.2007                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

switch flag

  case 0;                   % Initialization of S-function
   [sys,x0,str,ts]=mdlInitializeSizes(service,topic,Ts);
  
  case 2;                   % Data sending
    sys=mdlUpdate(t,x,u,item);
    
  case 9;                   % DDE communication closing
    sys=mdlTerminate(t,x,u);

  otherwise;
    sys=[];

end


function [sys,x0,str,ts]=mdlInitializeSizes(service,topic,Ts)
% Initialization of S-function

ts=[Ts 0];                  % Setting of sample time 
str=[];                     % Reserved for future use - muss be an empty matrix

sizes=simsizes;             % Loading information about the S-Function 

sizes.NumContStates=0;      % Number of continuous states
sizes.NumDiscStates=1;      % Number of discrete states
sizes.NumOutputs=0;         % Number of outputs
sizes.NumInputs=1;          % Number of inputs
sizes.DirFeedthrough=0;     % Flag for direct feedthrough
sizes.NumSampleTimes=1;     % Number of sample times

sys=simsizes(sizes);        % Saving of set information for Simulink

x0=ddeinit(service,topic);  % Initialization of DDE communication
                            % x0 contains channel handle
                            
if (x0==0)
  error('Initialization of DDE failed.');
end;


function sys=mdlUpdate(t,x,u,item)
% Data sending

ddepoke(x, item, u);        % Send data to application via DDE
sys=x;                      % DDE conversation channel handle


function sys=mdlTerminate(t,x,u)
% DDE communication closing

ddeterm(x);                 % DDE communication closing
sys=[];
